###############################################################################
# Default settings, which can be overriden by each individual project.
###############################################################################

# Shows debug messages while Silence is running
DEBUG_ENABLED = False

# The address and port in which the API and the web server will be deployed
LISTEN_ADDRESS = "0.0.0.0"  # Listen on all interfaces by default
HTTP_PORT = 8080

# The sequence of SQL scripts located in the sql/ folder that must
# be ran when the 'silence createdb' command is issued
SQL_SCRIPTS = []

# The URL prefix for all API endpoints
API_PREFIX = "/api"

# Enables or disables the API and the web server separately
RUN_API = True
RUN_WEB = True

# Database connection details
DB_CONN = {
    "host": "127.0.0.1",
    "port": 3306,
    "username": "default_username",
    "password": "default_password",
    "database": "default_database",
}

# Table and fields that are used for both login and register
USER_AUTH_DATA = {
    "table": "users",
    "identifier": "username",
    "password": "password",
}

# Enables or disables the /login and /register endpoints separately
ENABLE_LOGIN = True
ENABLE_REGISTER = True

# A random string that is used for signing session tokens and also passed to Flask
SECRET_KEY = "These are generated automatically for each project."

# Maximum validity time of a session token in seconds
MAX_TOKEN_AGE = 86400

# Default settings for creating a new project
PROJECT_TEMPLATE_NAME = "silence-app"
PROJECT_TEMPLATE_REPO = "https://github.com/IISSI-US/Silence-project-template.git"



import re
import zipfile
import requests

from os import listdir, remove, getcwd
from os.path import join, isfile, basename
from urllib.parse import urlparse
from shutil import move, rmtree
from secrets import token_urlsafe

# Adapted to our needs from https://github.com/x011/dload/
def save(url, path):
    r = requests.get(url)
    with open(path, 'wb') as f:
        f.write(r.content)
    return path

def git_clone(git_url, clone_dir):
    git_url = git_url.strip()
    if not git_url.lower().endswith(".git"):
        raise ValueError("Invalid Git URL")

    repo_name = re.sub(r"\.git$", "", git_url, 0, re.IGNORECASE | re.MULTILINE)
    repo_zip = repo_name + "/archive/master.zip"

    if isfile("master.zip"):
        remove("master.zip")

    fn = basename(urlparse(repo_zip).path)
    zip_path = save(repo_zip, join(getcwd(), fn))

    with zipfile.ZipFile(zip_path, 'r') as zip_ref:
        zip_ref.extractall(clone_dir)

    if isfile(zip_path):
        remove(zip_path)

def download_from_github(project_name, repo_url):
    git_clone(repo_url, project_name + "/")

    # Unpack it (everything is inside the <name>-master folder)
    branch_folder_name = listdir(project_name)[0]
    
    # Move everything inside that folder outside
    branch_folder = join(project_name, branch_folder_name)
    for elem in listdir(branch_folder):
        move(join(branch_folder, elem), join(project_name, elem))

    # Remove the now empty folder
    rmtree(branch_folder)

    # Generate the random string for the Flask secret key
    # and add it to the settings.py file
    secret_key = token_urlsafe(32)

    with open(join(project_name, "settings.py"), "a", encoding="utf-8") as f:
        f.write("\n\n")
        f.write("# A random string that is used for security purposes\n")
        f.write("# (this has been generated automatically upon project creation)\n")
        f.write(f'SECRET_KEY = "{secret_key}"\n')


from itsdangerous import URLSafeTimedSerializer as Serializer
from itsdangerous.exc import BadSignature, SignatureExpired

from silence.settings import settings
from silence.exceptions import TokenError
from silence.logging.default_logger import logger

###############################################################################
# Token management: creation and checking
###############################################################################

auth = Serializer(settings.SECRET_KEY)

def create_token(data):
    token = auth.dumps(data)
    logger.debug(f"Created new token {token[:6]}[...]{token[-6:]}")
    return token

def check_token(token):
    logger.debug(f"Checking received token {token[:6]}[...]{token[-6:]}")
    try:
        auth.loads(token, max_age=settings.MAX_TOKEN_AGE)
    except SignatureExpired:
        logger.debug("The token has expired")
        raise TokenError("The session token has expired")
    except BadSignature:
        logger.debug("The token is not valid")
        raise TokenError("The session token is not valid")

    logger.debug("The token is correct")


from pypika import Table, Field
from pypika import MySQLQuery as Query

def get_login_query(table_name, id_name, id_value):
    users = Table(table_name)
    q = Query.from_(users).select('*').where(users[id_name] == id_value)
    return str(q)

def get_register_user_query(table_name, user):
    field_names = tuple(k for k in user)
    field_values = tuple(user[k] for k in field_names)

    users = Table(table_name)
    q = Query.into(users).columns(*field_names).insert(*field_values)
    return str(q)



import importlib

from os import listdir
from os.path import splitext

from silence.server import manager, default_endpoints
from silence.settings import settings
from silence.logging.default_logger import logger

###############################################################################
# Look for .py files inside the project's "/api" folder
# and force them to run, activating the @endpoint decorators
###############################################################################

def load_user_endpoints():
    logger.debug("Looking for custom endpoints...")

    # Load every .py file inside the api/ folder
    for pyfile in listdir("api"):
        module_name = "api." + splitext(pyfile)[0]
        logger.debug(f"Found endpoint file: {module_name}")

        try:
            importlib.import_module(module_name)
        except ImportError:
            raise RuntimeError(f"Could not load the API file {module_name}")
    
###############################################################################
# Register the Silence-provided endpoints
###############################################################################
def load_default_endpoints():
    route_prefix = settings.API_PREFIX
    if route_prefix.endswith("/"):
        route_prefix = route_prefix[:-1]

    manager.API_TREE.register_endpoint({
            "route": route_prefix,
            "method": "GET",
            "desc": "Returns the data regarding the API endpoints",
    })
    manager.APP.add_url_rule(route_prefix, "APItreeHELP", default_endpoints.show_api_endpoints, methods=["GET"])

    if settings.ENABLE_LOGIN:
        login_route = f"{route_prefix}/login"
        manager.API_TREE.add_url(login_route)
        manager.API_TREE.register_endpoint({
            "route": login_route,
            "method": "POST",
            "desc": "Starts a new session, returning a session token and the user data if the login is successful",
        })
        manager.APP.add_url_rule(login_route, "login", default_endpoints.login, methods=["POST"])

    if settings.ENABLE_REGISTER:
        register_route = f"{route_prefix}/register"
        manager.API_TREE.add_url(register_route)
        manager.API_TREE.register_endpoint({
            "route": register_route,
            "method": "POST",
            "desc": "Creates a new user, returning a session token and the user data if the register is successful",
        })
        manager.APP.add_url_rule(register_route, "register", default_endpoints.register, methods=["POST"])
